import math


def 算数_求最小公倍数(a, b):
    """
    计算两个数的最小公倍数。

    参数:
        - a (int): 第一个整数。
        - b (int): 第二个整数。

    返回:
        - 最小公倍数 (int): 两个数的最小公倍数。
    """
    try:
        最小公倍数 = abs(a * b) // math.gcd(a, b)
        return 最小公倍数
    except Exception as e:
        print(f"计算最小公倍数时发生错误: {e}")
        return None


import json
import jsonpath


def json_文本数据查找(数据对象, 层级路径, 键, 值):
    """
    使用说明：
    1、数值数据查找，按照指定的层级目录下开始进行查找，提取所有符合条件的目标所在层级详细信息。

    2、层级名称不固定可用*号代替。遇到层级路径中如果包含列表就用索引选择[0]或[*]，比如：edges[0]，

    3、假设想要查找的数据在第五层内，那么层级路径只需要写到第四层（如果层级路径也写到第五层将失去查找的意义）

    参数:
    数据对象 (dict_字典 或 str_字符串): Python 字典对象或 json 字符串。
    层级路径 (str_字符串): 层级路径。
    键 (str_字符串): 要查找的键。
    值 (str_字符串): 要查找的值。

    返回:
    list: 匹配的数据列表，如果没有匹配或异常则返回空列表。

    使用示例：
    # 提取所有 'name' 为 'zeng' 的所在层级信息
    查找结果 = json_文本数据查找(数据对象, "data.第一层.第二层", "name", "zeng")
    print(查找结果)
    """
    try:
        # 如果传入的是 json 字符串，则解析为 Python 字典
        if isinstance(数据对象, str):
            数据对象 = json.loads(数据对象)

        # 构建 JSONPath 表达式，根据指定键和值查找匹配的数据
        表达式 = f"$.{层级路径}[?(@.{键} == '{值}')]"

        # 执行 JSONPath 查询
        匹配结果 = jsonpath.jsonpath(数据对象, 表达式)

        return 匹配结果 if 匹配结果 else []
    except Exception:
        return []


def 列表_排序(列表, 逆序=False):
    """
    返回排序后的列表。

    参数:
        列表 (list): 要排序的列表。
        逆序 (bool): 是否逆序排序，默认False（升序）。

    返回:
        list: 排序后的列表。

    示例:
        排序后的列表 = 列表_排序([3, 1, 2])
        if 排序后的列表 is not None:
            print("排序后的列表:", 排序后的列表)
        else:
            print("排序失败")
    """
    try:
        return sorted(列表, reverse=逆序)
    except Exception:
        return None


import subprocess
import sys
import os


def 模块_更新版本2():
    """
    更新名为 zfx 的 Python 模块。

    返回:
        bool: 如果更新成功，则返回True，否则返回False。

    示例:
        zfx.模块_更新版本2()
    """
    try:
        # 获取当前Python解释器的目录
        python_dir = os.path.dirname(sys.executable)
        pip_path = os.path.join(python_dir, 'Scripts', 'pip.exe')

        # 清理 pip 缓存
        subprocess.run([pip_path, "cache", "purge"], check=True)
        # 更新 zfx 模块
        subprocess.run([pip_path, "install", "--upgrade", "zfx"], check=True)
        return True
    except Exception:
        return False  # 捕获其他所有异常并返回 False


def 文本文件_读入全部内容(文件路径):
    """
    读取指定文件路径的文本文件内容。
    注意：文本编码格式需要为UTF-8

    参数:
        文件路径 (str): 要读取的文本文件的路径。

    返回:
        str: 文本文件的内容，失败将返回 None。

    示例:
        文件内容 = 文本文件_读入全部内容('示例.txt')
        if 文件内容 is not None:
            print("文件内容:", 文件内容)
        else:
            print("读取文件失败")
    """
    try:
        with open(文件路径, 'r', encoding='utf-8') as file:
            文件内容 = file.read()
        return 文件内容
    except Exception:
        return None



import math


def 算数_求余弦(角度):
    """
    计算给定角度的余弦值。

    参数:
        - 角度 (float): 要计算余弦值的角度，单位为度。

    返回:
        - 余弦值 (float): 给定角度的余弦值。如果输入无效或出现异常，返回 None 并打印错误信息。

    # 示例用法
    angle = 60
    cos_value = 算数_求余弦(angle)
    print(f"{angle}度的余弦值为: {cos_value}")
    """
    try:
        # 将角度转换为弧度
        弧度 = math.radians(角度)

        # 计算余弦值
        余弦值 = math.cos(弧度)

        return 余弦值
    except Exception:
        return None


def 文本_从中间往左取(字符串, 开始位置, 字符数量):
    """
    从字符串的指定开始位置向左提取指定数量的字符。

    参数:
        - 字符串 (str): 输入的字符串。
        - 开始位置 (int): 开始提取的起始位置（从0开始）。
        - 字符数量 (int): 要提取的字符数量。

    返回:
        - str: 从指定位置向左提取的字符。如果字符数量大于开始位置，则返回从字符串开头到开始位置的所有字符。
        - bool: 如果出现异常，返回 False。
    """
    try:
        if not isinstance(字符串, str) or not isinstance(开始位置, int) or not isinstance(字符数量, int):
            return False

        if 开始位置 < 0 or 字符数量 < 0 or 开始位置 >= len(字符串):
            return False

        return 字符串[max(0, 开始位置 - 字符数量):开始位置]
    except Exception:
        return False


def 谷歌填表_前进(驱动器对象):
    """
    使用提供的驱动程序执行前进操作。

    参数:
        - 驱动器对象: WebDriver 对象，用于控制浏览器的行为。

    返回:
        - bool: 前进操作成功返回 True，失败返回 False。
    """
    try:
        驱动器对象.forward()
        return True
    except Exception:
        return False



def 列表_取平均值(列表):
    """
    返回列表中所有元素的平均值。

    参数:
        列表 (list): 包含数字的列表。

    返回:
        数值: 列表中所有元素的平均值。如果计算过程中发生异常，则返回错误信息。

    示例:
        平均值 = 列表_取平均值([1, 2, 3])
        if 平均值 is not None:
            print("列表元素的平均值:", 平均值)
        else:
            print("计算平均值失败")
    """
    try:
        # 检查列表是否为空
        if not 列表:
            return None

        # 计算并返回列表的平均值
        平均值 = sum(列表) / len(列表)
        return 平均值
    except Exception as e:
        # 捕获所有异常并返回错误信息
        return f"处理失败: {e}"


def 邮件POP3_获取最新一封邮件(服务器对象):
    """
    从已连接的POP3服务器中获取最新的一封邮件。

    参数:
        - 邮件服务器 (poplib.POP3): 已连接并登录的POP3服务器对象。

    返回值:
        - 如果获取成功，返回最新邮件的原始内容（字节序列）；如果获取失败或出现异常，返回假(False)。
    """
    try:
        # 获取邮件数量
        邮件数量, 邮件总大小 = 服务器对象.stat()

        if 邮件数量 == 0:
            return False  # 如果邮箱中没有邮件，返回 False

        # 获取最新的一封邮件（编号从1开始）
        最新邮件编号 = 邮件数量
        响应状态, 邮件内容, 邮件字节数 = 服务器对象.retr(最新邮件编号)

        # 将邮件内容组合成一个字节序列
        邮件完整内容 = b'\r\n'.join(邮件内容)

        return 邮件完整内容  # 直接返回原始邮件内容（字节序列）
    except Exception:
        return False  # 如果获取邮件失败，返回 False


import math


def 算数_求阶乘(值):
    """
    计算给定整数的阶乘。

    参数:
        - 值 (int): 要计算的整数。

    返回:
        - 阶乘值 (int): 给定整数的阶乘。
    """
    try:
        阶乘值 = math.factorial(值)
        return 阶乘值
    except Exception as e:
        print(f"计算阶乘值时发生错误: {e}")
        return None


def 列表_元素替换(列表, 旧元素, 新元素):
    """
    将列表中的某个元素替换为另一个元素。

    参数:
        - 列表 (list): 需要处理的列表
        - 旧元素 (any): 需要替换的元素
        - 新元素 (any): 用于替换的元素

    返回:
        - 新列表 (list): 替换后的列表。如果处理过程中发生异常，则返回空列表。
    """
    try:
        # 使用列表推导式创建新列表
        新列表 = [新元素 if x == 旧元素 else x for x in 列表]
        return 新列表
    except Exception as e:
        # 捕获异常并返回错误信息
        return []


import pyperclip


def 系统_取剪辑版内容():
    """
    获取系统剪辑板的内容。

    返回:
        str: 系统剪辑板的内容。如果获取失败，则返回 None。

    使用示例:
        剪辑版内容 = 系统_取剪辑版内容()
        print("剪辑版内容:", 剪辑版内容)
    """

    try:
        剪辑版内容 = pyperclip.paste()
        return 剪辑版内容
    except Exception:
        return None


def 列表_筛选_最大长度过滤(列表, 最大长度):
    """
    过滤掉列表中长度大于指定值的元素，并返回过滤后的新列表。

    参数:
        - 列表 (list): 包含字符串或其他序列类型元素的列表。
        - 最大长度 (int): 保留元素的最大长度。

    返回:
        - 过滤后的新列表 (list): 只包含长度小于等于指定值的元素的列表。如果处理过程中发生异常，则返回错误信息。

    示例:
        结果 = 列表_筛选_最大长度过滤(["apple", "banana", "cherry"], 5)
        print("过滤后的新列表:", 结果)
    """
    try:
        # 使用列表推导式过滤掉长度大于最大长度的元素
        过滤后的新列表 = [元素 for 元素 in 列表 if len(元素) <= 最大长度]
        return 过滤后的新列表
    except Exception as e:
        # 捕获所有异常并返回错误信息
        return f"处理失败: {e}"


import threading


def 线程_创建新线程(目标函数, 线程名称=None, *参数):
    """
    创建一个新线程并启动。

    参数:
        目标函数 (callable): 线程要执行的目标函数
        线程名称 (str, optional): 线程的名称，默认为 None
        *参数: 传递给目标函数的参数

    返回:
        threading.Thread: 创建的线程对象，如果创建失败则返回 None
    """
    try:
        线程 = threading.Thread(target=目标函数, args=参数, name=线程名称)
        线程.start()
        return 线程
    except Exception:
        return None


import math


def 算数_求平方根(数字):
    """
    计算给定数字的平方根。

    参数:
        - 数字: 要计算平方根的数字。

    返回:
        - 数字，给定数字的平方根。如果输入无效或出现异常，返回 None 并打印错误信息。

    # 调用示例
    result = 求平方根(16)
    print(result)  # 输出结果为 4.0


    """
    try:
        result = math.sqrt(数字)
        return result
    except Exception as e:
        print(f"计算平方根时发生错误: {e}")
        return None


def 文本文件_插入内容(文件路径, 行号, 内容):
    """
    在指定行前插入内容。

    参数:
        - 文件路径 (str): 要操作的文件路径。
        - 行号 (int): 插入内容的位置（行号）。
        - 内容 (str): 要插入的内容。

    返回:
        - bool: 如果插入成功，返回 True；否则返回 False。
    """
    try:
        with open(文件路径, 'r+', encoding='utf-8') as file:
            行列表 = file.readlines()
            行列表.insert(行号 - 1, 内容 + '\n')
            file.seek(0)
            file.writelines(行列表)
        return True
    except Exception:
        return False



import math


def 算数_求幂(基数, 指数):
    """
    计算给定数值的幂。

    参数:
        - 基数 (float): 要计算的基数。
        - 指数 (float): 要计算的指数。

    返回:
        - 幂值 (float): 给定数值的幂。
    """
    try:
        幂值 = math.pow(基数, 指数)
        return 幂值
    except Exception as e:
        print(f"计算幂值时发生错误: {e}")
        return None


def 谷歌填表_模拟点击(驱动器对象, 元素对象):
    """
    模拟点击元素对象。

    参数:
        - 驱动器对象: WebDriver 对象，用于执行 JavaScript 脚本。
        - 元素对象: 要模拟点击的 WebElement 对象。

    返回:
        - bool: 成功返回 True，失败返回 False。
    """
    try:
        驱动器对象.execute_script("arguments[0].click();", 元素对象)
        return True
    except Exception:
        return False


def 谷歌填表_点击元素(元素对象):
    """
    点击指定的网页元素。

    参数:
        - 元素对象: 要点击的 WebElement 对象。

    返回:
        - bool: 如果点击成功，则返回 True，否则返回 False。
    """
    try:
        元素对象.click()
        return True
    except Exception:
        return False


def 文本_取文本最后出现的位置(文本, 查找的文本):
    """
    返回指定文本中指定子串最后一次出现的位置。

    参数：
    文本 (str): 要搜索的文本。
    查找的文本 (str): 要查找的文本。

    返回值：
    int: 最后一次出现的位置索引，如果没有找到或者出现异常则返回-1。
    """
    try:
        # 使用rfind函数查找子串最后一次出现的位置
        最后位置 = 文本.rfind(查找的文本)
        return 最后位置
    except Exception:
        return -1


def 算数_求和(数值列表):
    """
    计算一组数的和。

    参数:
        - 数值列表 (list): 要计算和的数值列表。

    返回:
        - 和 (float): 数值列表的和。
    """
    try:
        和 = sum(数值列表)
        return 和
    except Exception as e:
        print(f"计算和时发生错误: {e}")
        return None


import os


def 目录_取system32目录(加尾斜杠=False):
    """
    获取系统中 System32 目录的路径。

    参数:
        - 添加反斜杠: bool, 是否在返回的路径末尾添加反斜杠，默认为 False。

    返回:
        str: System32 目录的路径，如果操作失败则返回 False。
    """
    try:
        # 使用 os.environ 获取环境变量中的 SystemRoot
        system_root = os.environ.get('SystemRoot', 'C:\\Windows')
        # 构造 System32 目录的完整路径
        system32_path = os.path.join(system_root, 'System32')

        # 检查路径是否存在，并且确保是一个目录
        if os.path.exists(system32_path) and os.path.isdir(system32_path):
            if 加尾斜杠:
                return system32_path + '\\'
            else:
                return system32_path
        else:
            return False
    except Exception:
        return False


import subprocess
import sys
import os


def 模块_查看版本信息2():
    """
    查看模块 zfx 的版本号，及简介。

    返回:
        str: 模块 zfx 的版本号，如果模块不存在或发生异常，则打印失败信息。

    示例:
        zfx.模块_查看版本信息2()
    """
    try:
        # 获取当前Python解释器的目录
        python_dir = os.path.dirname(sys.executable)
        pip_path = os.path.join(python_dir, 'Scripts', 'pip.exe')

        # 使用subprocess模块执行固定的命令
        subprocess.run([pip_path, "show", "zfx"], check=True)
    except Exception:
        print(f"版本信息获取失败，请稍后再试")


def mysql_查询_取所有表名(连接对象):
    """
    获取数据库中所有表名。

    参数：
        - 连接对象：与数据库的连接对象。

    返回值：
        - 表名列表：包含所有表名的列表，如果查询失败则返回 None。
    """
    游标对象 = None
    try:
        # 创建游标对象，用于执行SQL查询
        游标对象 = 连接对象.cursor()
        # 执行SQL查询，获取所有表名
        游标对象.execute("SHOW TABLES")
        # 将查询结果转换为列表
        表名列表 = [row[0] for row in 游标对象.fetchall()]
        return 表名列表
    except Exception:
        return None
    finally:
        # 无论是否发生异常，关闭游标对象以释放资源
        if 游标对象:
            游标对象.close()


import mysql.connector


def mysql_连接数据库(主机, 用户名, 密码, 数据库名):
    """
    连接到MySQL数据库并返回 连接对象, 游标对象。

    参数：
        - 主机：数据库主机名或IP地址。
        - 用户名：数据库用户名。
        - 密码：数据库密码。
        - 数据库名：要连接的数据库名。

    返回值：
        - 连接对象：表示与数据库的连接。如果连接失败，则返回 False。
        - 游标对象：用于执行查询和获取结果。如果连接失败，则返回 False。
    """
    try:
        # 连接到数据库
        连接对象 = mysql.connector.connect(
            host=主机,
            user=用户名,
            password=密码,
            database=数据库名
        )

        # 创建游标对象
        游标对象 = 连接对象.cursor()

        # 返回连接对象和游标对象
        return 连接对象, 游标对象
    except Exception:
        return False, False


def 邮件POP3_获取最新五封邮件(服务器对象):
    """
    从已连接的POP3服务器中获取最新的五封邮件，以列表形式返回。

    参数:
        - 服务器对象 (poplib.POP3): 已连接并登录的POP3服务器对象。

    返回值:
        - 如果获取成功，返回最新五封邮件的原始内容列表（字节序列）；如果获取失败或出现异常，返回假(False)。
    """
    try:
        # 获取邮件数量
        邮件数量, 邮件总大小 = 服务器对象.stat()

        if 邮件数量 == 0:
            return False  # 如果邮箱中没有邮件，返回 False

        # 确定要获取的邮件数量（最多获取五封邮件）
        获取邮件数量 = min(5, 邮件数量)

        # 初始化邮件列表
        邮件列表 = []

        # 获取最新的五封邮件（编号从1开始）
        for i in range(邮件数量, 邮件数量 - 获取邮件数量, -1):
            响应状态, 邮件内容, 邮件字节数 = 服务器对象.retr(i)
            邮件完整内容 = b'\r\n'.join(邮件内容)
            邮件列表.append(邮件完整内容)

        return 邮件列表  # 返回最新五封邮件的原始内容列表（字节序列）
    except Exception:
        return False  # 如果获取邮件失败，返回 False


def mysql_查询_表结构(连接对象, 表名):
    """
    导出指定表的结构信息：字段名、数据类型、长度、是否允许为 NULL、是否为主键、是否自增等，获得的 表结构信息 可直接用于创建表单。

    参数：
        - 连接对象：MySQL 数据库连接对象。
        - 表名：要导出结构的表名。

    返回值：
        - 表结构信息：返回一个列表，每个元素为一个字典，包含每个字段的详细信息，如字段名、数据类型、长度、是否允许为空、是否为主键、是否自增等。
          如果获取结构失败，返回空列表。
    """
    表结构信息列表 = []

    try:
        # 查询表结构信息
        查询字段信息 = f"SELECT COLUMN_NAME, COLUMN_TYPE, CHARACTER_MAXIMUM_LENGTH, IS_NULLABLE, COLUMN_KEY, EXTRA " \
                     f"FROM information_schema.COLUMNS " \
                     f"WHERE TABLE_SCHEMA = '{连接对象.database}' AND TABLE_NAME = '{表名}'"

        cursor = 连接对象.cursor()

        # 获取字段信息
        cursor.execute(查询字段信息)
        字段结果 = cursor.fetchall()

        # 遍历每个字段的信息
        for row in 字段结果:
            字段名 = row[0]
            列类型 = row[1]
            if '(' in 列类型 and 列类型.endswith(')'):
                数据类型, 长度 = 列类型.split('(')
                长度 = 长度[:-1]
            else:
                数据类型, 长度 = 列类型, None
            是否允许为空 = row[3]
            是否主键 = (row[4] == 'PRI')
            是否自增 = ('auto_increment' in row[5])

            # 将字段信息添加到列表中
            表结构信息列表.append({
                "字段名": 字段名,
                "数据类型": 数据类型,
                "长度": 长度,
                "是否允许为空": 是否允许为空,
                "是否主键": 是否主键,
                "是否自增": 是否自增
            })

        return 表结构信息列表

    except Exception:
        return []


from bs4 import BeautifulSoup


def html_提取所有指定标签(html文本, 标签名):
    """
    从给定的HTML文本中提取所有指定的标签，并返回包含这些标签的列表。

    参数:
        html文本 (str): HTML 文本
        标签名 (str): 要提取的标签名

    返回:
        list: 包含所有指定标签的列表

    常用标签名示例:
        - 'a': 超链接
        - 'p': 段落
        - 'div': 区块
        - 'span': 行内区块
        - 'img': 图像
        - 'ul': 无序列表
        - 'ol': 有序列表
        - 'li': 列表项
        - 'table': 表格
        - 'tr': 表格行
        - 'td': 表格单元格
        - 'th': 表格表头
        - 'form': 表单
        - 'input': 输入框
        - 'button': 按钮
        - 'label': 标签
        - 'header': 页头
        - 'footer': 页脚
        - 'nav': 导航
        - 'section': 章节
        - 'article': 文章
        - 'aside': 旁注内容
        - 'iframe': 内联框架
        - 'h1', 'h2', 'h3', 'h4', 'h5', 'h6': 标题
        - 'meta': 元数据
        - 'link': 链接到外部资源
        - 'script': 脚本
    """
    try:
        soup = BeautifulSoup(html文本, 'html.parser')
        标签列表 = soup.find_all(标签名)
        return 标签列表
    except Exception:
        return []


def 数据类型转换_到元组(输入值):
    """
    将输入值转换为元组。

    参数:
    输入值: 任何类型
        需要转换为元组的值。

    返回:
    tuple
        输入值转换后的元组。如果转换失败，则返回错误信息。
    """
    try:
        return tuple(输入值)
    except TypeError:
        # 如果输入值无法直接转换为元组，尝试将其包装在一个元组中
        return (输入值,)
    except Exception as e:
        return e


import platform


def 系统_取操作系统类别():
    """
    获取操作系统类别。

    返回:
        str: 当前操作系统类别，例如 "Windows 10"、"Windows 7"。如果获取失败，则返回 "未知版本"。

    使用示例:
        os_category = 系统_取操作系统类别()
        print("操作系统类别:", os_category)
    """
    try:
        system = platform.system()
        version = platform.version()

        if "Windows" in system:
            # Windows 操作系统
            if "XP" in version:
                return "Windows XP"
            elif "2003" in version:
                return "Windows Server 2003"
            elif "Vista" in version:
                return "Windows Vista"
            elif "7" in version:
                return "Windows 7"
            elif "8.1" in version:
                return "Windows 8.1"
            elif "8" in version or "2012" in version:
                return "Windows 8/Windows Server 2012"
            elif "10" in version:
                return "Windows 10"
            else:
                return "未知版本"  # 未知版本
        else:
            return "非 Windows 系统"  # 非 Windows 系统，返回未知版本
    except Exception:
        return "未知版本"


def 文本_取出中间文本(欲取全文本, 前面文本, 后面文本, 起始搜寻位置=None, 是否不区分大小写=False):
    """
    从全文本中提取位于前文本和后文本之间的中间文本。

    参数:
        欲取全文本 (str): 要搜索的全文本。
        前面文本 (str): 中间文本的前部分。
        后面文本 (str): 中间文本的后部分。
        起始搜寻位置 (int): 可选参数，指定搜索起始位置，默认为 None。
        是否不区分大小写 (bool): 是否不区分大小写，默认为 False。

    返回:
        str: 提取出的中间文本。如果出现任何异常或找不到指定的前后文本，则返回空字符串。

    示例:
        全文本 = "这是一个示例文本，中间包含了要取出的部分。"
        前文本 = "示例文本，中间包含了"
        后文本 = "部分。"
        中间文本 = 文本_取出中间文本(全文本, 前文本, 后文本)
        print("取出的中间文本:", 中间文本)
    """
    try:
        if 是否不区分大小写:
            欲取全文本 = 欲取全文本.lower()
            前面文本 = 前面文本.lower()
            后面文本 = 后面文本.lower()

        if 起始搜寻位置 is not None:
            起始索引 = 欲取全文本.find(前面文本, 起始搜寻位置)
        else:
            起始索引 = 欲取全文本.find(前面文本)

        if 起始索引 == -1:
            return ""

        起始索引 += len(前面文本)
        结束索引 = 欲取全文本.find(后面文本, 起始索引)
        if 结束索引 == -1:
            return ""

        return 欲取全文本[起始索引:结束索引]
    except Exception:  # 捕获所有异常
        return ""


def mysql_更新_字典条件(连接, 表名, 值字典, 条件字典=None):
    """
    更新指定字段的数据。

    参数：
        - 连接: 数据库连接对象。
        - 表名：要更新数据的表的名称。
        - 值字典：要更新的字段及其对应的值，字典类型。例如 {"字段1": "值1", "字段2": "值2", "字段3": 1234}。
        - 条件字典：查找符合条件的记录的条件，字典类型。例如 {"id": 1, "name": "Jack"}。如果为空字典或为 None，则更新所有记录。

    返回值：
        - 更新成功返回 True，失败返回 False。
    """
    try:
        # 构建更新语句
        赋值语句 = ', '.join([f"{字段}=%s" for 字段 in 值字典.keys()])
        更新语句 = f"UPDATE {表名} SET {赋值语句}"

        # 如果有条件，则添加条件到更新语句中
        if 条件字典:
            条件语句 = ' AND '.join([f"{字段}=%s" for 字段 in 条件字典.keys()])
            更新语句 += f" WHERE {条件语句}"

        # 提取值列表
        值列表 = list(值字典.values())
        if 条件字典:
            值列表.extend(list(条件字典.values()))

        # 执行更新操作
        连接.cursor().execute(更新语句, 值列表)

        # 提交事务
        连接.commit()

        return True
    except Exception:
        return False


def mysql_删除记录(连接, 表名, 条件):
    """
    删除符合条件的记录。

    参数：
        - 连接: 数据库连接对象。
        - 表名：要进行删除操作的表的名称。
        - 条件：删除符合条件的记录的条件。如果为''(空字符串)，则删除所有记录。

    返回值：
        - 删除成功返回 True，失败返回 False。

    使用示例
    删除结果 = 数据库_mysql_删除记录(连接对象,  "asida", "username='jack'")
    删除结果 = 数据库_mysql_删除记录(连接对象,  "asida", "id > 5")
    """
    try:
        # 构建删除语句
        删除语句 = f"DELETE FROM {表名}"
        if 条件:
            删除语句 += f" WHERE {条件}"

        # 执行删除操作
        连接.cursor().execute(删除语句)

        # 提交事务
        连接.commit()

        return True
    except Exception:
        return False


def 列表_去除指定元素(列表, 元素):
    """
    返回去除指定元素后的列表。

    参数:
        列表 (list): 原始列表。
        元素 (any): 要去除的元素。

    返回:
        list: 去除指定元素后的列表。

    示例:
        新列表 = 列表_去除指定元素([1, 2, 3, 2], 2)
        if 新列表 is not None:
            print("去除指定元素后的列表:", 新列表)
        else:
            print("去除指定元素失败")
    """
    try:
        return [x for x in 列表 if x != 元素]
    except Exception:
        return None


import poplib


def 邮件POP3_登录(服务器地址, 用户名, 密码):
    """
    尝试连接到指定的POP3服务器并登录邮箱账户(只支持 110 端口的协议)。

    参数:
        - 服务器地址 (str): POP3服务器的地址。
        - 用户名 (str): 登录邮箱的用户名。
        - 密码 (str): 登录邮箱的密码。

    返回值:
        - 如果登录成功，返回连接成功的POP3服务器对象；如果登录失败或出现异常，返回假(False)。
    """
    try:
        # 连接到POP3服务器
        邮件服务器 = poplib.POP3(服务器地址, 110)

        # 尝试登录邮箱
        响应用户 = 邮件服务器.user(用户名)
        响应密码 = 邮件服务器.pass_(密码)

        # 判断登录是否成功
        if 响应用户.startswith(b'+OK') and 响应密码.startswith(b'+OK'):
            return 邮件服务器  # 返回连接成功的POP3服务器对象
    except Exception:
        return False  # 如果连接失败或登录失败，直接返回 False


def 列表_去除重复(列表):
    """
    去除列表中的重复元素，并返回去重后的新列表。

    参数:
        - 列表 (list): 包含元素的列表，可以是数字，字符串

    返回:
        - 去重后的新列表 (list): 去除重复元素后的列表。如果处理过程中发生异常，则返回错误信息。
    """
    try:
        # 使用 set 去除重复元素，并将结果转换回列表
        去重后的新列表 = list(set(列表))
        return 去重后的新列表
    except Exception as e:
        # 捕获所有异常并返回错误信息
        return f"处理失败: {e}"


def 数据类型转换_到列表(输入值):
    """
    将输入值转换为列表(数组)。

    参数:
    输入值: 任何类型
        需要转换为列表的值。

    返回:
    list
        输入值转换后的列表。如果转换失败，则返回错误信息。
    """
    try:
        return list(输入值)
    except TypeError:
        # 如果输入值无法直接转换为列表，尝试将其包装在一个列表中
        return [输入值]
    except Exception as e:
        return e


def 网页协议_响应对象_取中间(响应对象, 文本前缀, 文本后缀):
    """
    获取响应的文本内容，并从中提取介于文本前缀和文本后缀之间的部分文本。

    参数:
        - 响应对象: 服务器响应对象。
        - 文本前缀: 前缀字符串，用于定位目标文本的起始位置。
        - 文本后缀: 后缀字符串，用于定位目标文本的结束位置。

    返回:
        - str: 响应文本中位于文本前缀和文本后缀之间的部分文本。如果响应为 None、未找到匹配的文本、或出现任何异常，则返回空字符串。
    """
    try:
        if 响应对象 is not None:
            文本内容 = 响应对象.text
            起始位置 = 文本内容.find(文本前缀)
            if 起始位置 != -1:
                起始位置 += len(文本前缀)
                结束位置 = 文本内容.find(文本后缀, 起始位置)
                if 结束位置 != -1:
                    提取的文本 = 文本内容[起始位置:结束位置]
                    return 提取的文本
            return ''
        else:
            return ''
    except Exception:
        return ''


def 列表_并集(列表1, 列表2):
    """
    返回两个列表的并集(将两个列表合并，想同的元素只会出现一次)。

    参数:
        - 列表1 (list): 第一个列表。
        - 列表2 (list): 第二个列表。

    返回:
        - list: 包含两个列表中所有不同元素的列表。如果出现异常，返回空列表。
    """
    try:
        # 使用集合求并集，然后转换回列表
        并集结果 = list(set(列表1) | set(列表2))
        return 并集结果
    except Exception:
        return []


import math


def 算数_求正弦(角度):
    """
    计算给定角度的正弦值。

    参数:
        - 角度 (float): 要计算正弦值的角度，单位为度。

    返回:
        - 正弦值 (float): 给定角度的正弦值。如果输入无效或出现异常，返回 None 并打印错误信息。

    示例用法:
        angle = 45
        sin_value = 算数_求正弦(angle)
        print(f"{angle}度的正弦值为: {sin_value}")
    """
    try:
        # 将角度转换为弧度
        弧度 = math.radians(角度)

        # 计算正弦值
        正弦值 = math.sin(弧度)

        return 正弦值
    except Exception as e:
        print(f"计算正弦值时发生错误: {e}")
        return None


def 文本文件_取指定行内容(文件路径, 行号):
    """
    获取指定文本文件中特定行的内容。
    注意：文本编码格式需要为UTF-8

    参数:
        - 文件路径 (str): 要读取的文本文件的路径。
        - 行号 (int): 要获取内容的行号（从1开始）。

    返回:
        - str: 指定行的内容。失败返回 None。

    示例:
        行内容 = 文本文件_取指定行内容('示例.txt', 3)
        if 行内容 is not None:
            print("行内容:", 行内容)
        else:
            print("获取行内容失败")
    """
    try:
        with open(文件路径, 'r', encoding='utf-8') as file:
            行内容 = file.readlines()

        if 行号 < 1 or 行号 > len(行内容):
            return None

        return 行内容[行号 - 1].rstrip('\n')
    except Exception:
        return None


def 列表_去除前后空格(列表):
    """
    去除列表中每个字符串元素的前后空格。

    参数:
        - 列表 (list): 要处理的列表。

    返回:
        - list: 去除前后空格后的新列表。如果出现异常，返回原始列表。
    """
    try:
        # 使用列表推导式去除每个字符串元素的前后空格
        新列表 = [元素.strip() if isinstance(元素, str) else 元素 for 元素 in 列表]
        return 新列表
    except Exception:
        return 列表


import math


def 算数_求最大公约数(a, b):
    """
    计算两个数的最大公约数。

    参数:
        - a (int): 第一个整数。
        - b (int): 第二个整数。

    返回:
        - 最大公约数 (int): 两个数的最大公约数。
    """
    try:
        最大公约数 = math.gcd(a, b)
        return 最大公约数
    except Exception as e:
        print(f"计算最大公约数时发生错误: {e}")
        return None


def 文本_取右边(被查找的文本, 欲寻找的文本, 起始寻找位置=0, 是否不区分大小写=False):
    """
    从被查找的文本中寻找指定的文本，并返回该文本右边的所有字符。如果找不到，返回空字符串。

    参数:
        - 被查找的文本 (str): 要在其中进行查找的文本。
        - 欲寻找的文本 (str): 需要寻找的目标文本。
        - 起始寻找位置 (int, optional): 开始查找的位置索引，默认为 0。
        - 是否不区分大小写 (bool, optional): 如果为 True，查找时不区分大小写；如果为 False，区分大小写。默认值为 False。

    返回:
        - str: 欲寻找的文本右边的文本内容。如果找不到或出现异常，返回空字符串。

    示例用法:
        被查找的文本 = "这是一个示例文本，Python是一种很强大的编程语言"
        欲寻找的文本 = "Python"
        结果 = 文本_取右边(被查找的文本, 欲寻找的文本, 1, True)
        print(结果)  # 输出：是一种很强大的编程语言
    """
    try:
        if 是否不区分大小写:
            被查找的文本 = 被查找的文本.lower()
            欲寻找的文本 = 欲寻找的文本.lower()

        寻找位置 = 被查找的文本.find(欲寻找的文本, 起始寻找位置)
        if 寻找位置 != -1:
            return 被查找的文本[寻找位置 + len(欲寻找的文本):]
        else:
            return ""
    except Exception:
        return ""


def 数据类型转换_到整数(输入值):
    """
    将输入值转换为整数。

    参数:
    输入值: 任何类型
        需要转换为整数的值。

    返回:
    int
        输入值转换后的整数。如果转换失败，则返回错误信息。
    """
    try:
        return int(输入值)
    except Exception as e:
        return e


def 算数_求次方(基数, 指数):
    """
    计算基数的指定次方。

    参数:
        - 基数 (int or float): 要进行幂运算的基数。
        - 指数 (int or float): 要进行幂运算的指数。

    返回:
        - 数字 (int or float): 计算结果，即基数的指定次方。如果输入无效或出现异常，返回 None 并打印错误信息。

    使用示例:
        result = 算数_求次方(2, 3)
        print(result)  # 输出结果为 8

        result = 算数_求次方(2.5, 2)
        print(result)  # 输出结果为 6.25
    """
    try:
        result = 基数 ** 指数
        return result
    except Exception as e:
        print(f"计算次方时发生错误: {e}")
        return None


import datetime


def 系统_取时区():
    """
    获取系统当前时区。

    返回:
        str: 系统当前时区信息。如果获取失败，则返回 None。

    使用示例:
        timezone = 系统_取时区()
        print("系统时区:", timezone)
    """
    try:
        # 获取当前日期时间对象
        current_time = datetime.datetime.now()
        # 获取当前时区信息
        timezone_info = current_time.astimezone().tzinfo
        # 格式化时区信息为字符串
        timezone_str = str(timezone_info)
        return timezone_str
    except Exception:
        return None


def 列表_交集(列表1, 列表2):
    """
    返回两个列表的交集(两个列表都有的元素，就叫交集)。

    参数:
        - 列表1 (list): 第一个列表。
        - 列表2 (list): 第二个列表。

    返回:
        - list: 包含两个列表中共同元素的列表。如果出现异常，返回空列表。
    """
    try:
        # 使用集合求交集，然后转换回列表
        交集结果 = list(set(列表1) & set(列表2))
        return 交集结果
    except Exception:
        return []


import subprocess
import sys
import os


def 模块_更新到指定版本2(version):
    """
    更新名为 zfx 的 Python 模块到指定版本。

    参数:
        version (str): 要更新到的版本号。

    返回:
        bool: 如果更新成功，则返回True，否则返回False。

    示例:
        zfx.模块_更新到指定版本2()
    """
    try:
        # 获取当前Python解释器的目录
        python_dir = os.path.dirname(sys.executable)
        pip_path = os.path.join(python_dir, 'Scripts', 'pip.exe')

        # 使用subprocess模块执行固定的命令
        subprocess.run([pip_path, "install", "--upgrade", f"zfx=={version}"], check=True)
        return True
    except Exception:
        return False  # 捕获其他所有异常并返回 False


def 谷歌填表_执行JavaScript(驱动器对象, 脚本, *参数):
    """
    执行 JavaScript 脚本。

    参数:
        - 驱动器对象: WebDriver 对象。
        - 脚本: 要执行的 JavaScript 脚本。
        - *参数: 传递给 JavaScript 脚本的参数。

    返回:
        - 执行 JavaScript 脚本后的返回值。失败返回 None。
    """
    try:
        return 驱动器对象.execute_script(脚本, *参数)
    except Exception:
        return None


def 文本_到大写(要转换的文本):
    """
    将文本转换为大写。

    参数:
        要转换的文本 (str): 要转换为大写的文本。

    返回:
        str: 转换为大写的文本。如果转换失败或出现任何异常，则返回 False。

    示例:
        文本 = "Hello world"
        结果 = 文本_到大写(文本)
        print(结果)  # 输出：HELLO WORLD
    """
    try:
        大写文本 = 要转换的文本.upper()
        return 大写文本
    except Exception:  # 捕获所有异常
        return False


def 文本_去除指定字符串_首次出现(原始文本: str, 目标文本: str):
    """
    去除原始文本中目标文本的首次出现。

    参数:
        - 原始文本(str): 原始的文本内容。
        - 目标文本(str): 需要去除的字符或字符串。

    返回:
        处理后的文本。如果出现异常则返回原始文本
    """
    try:
        # 查找目标文本首次出现的位置
        位置 = 原始文本.find(目标文本)
        # 如果找到了目标文本，去除它的首次出现
        if 位置 != -1:
            return 原始文本[:位置] + 原始文本[位置 + len(目标文本):]
        # 如果没有找到目标文本，返回原始文本
        return 原始文本
    except Exception:
        return 原始文本


import os


def 文件_取大小B(文件完整路径):
    """
    获取指定文件的大小（以字节为单位）。

    参数:
    文件完整路径 (str): 要获取大小的文件路径。

    返回:
    int: 文件的大小（字节）,失败则返回None
    """
    try:
        return os.path.getsize(文件完整路径)
    except Exception:
        return None


def 列表_移除偶数索引元素(列表):
    """
    移除列表中所有偶数索引位置的元素。

    参数:
        - 列表 (list): 要处理的列表。

    返回:
        - list: 移除偶数索引元素后的新列表。如果出现异常，返回原始列表。
    """
    try:
        # 使用列表推导式生成新列表，保留奇数索引位置的元素
        新列表 = [元素 for 索引, 元素 in enumerate(列表) if 索引 % 2 != 0]
        return 新列表
    except Exception:
        return 列表


def 邮件IMAP_登录邮箱(连接对象, 用户名, 密码):
    """
    登录到IMAP邮箱账户。

    参数:
        邮箱 (IMAP4_SSL): IMAP4_SSL连接对象。
        用户名 (str): 邮箱用户名。
        密码 (str): 邮箱密码。

    返回:
        bool: 登录成功返回True，否则返回False。
    """
    try:
        连接对象.login(用户名, 密码)
        return True
    except Exception:
        return False


def 谷歌填表_取当前页面网址(驱动器对象):
    """
    获取当前页面的URL。

    参数:
        驱动器对象: 已经初始化的Selenium Chrome驱动对象。

    返回:
        str: 当前页面的URL，获取失败则返回None。
    """
    try:
        return 驱动器对象.current_url
    except Exception:
        return None


def 谷歌填表_判断可用(元素对象):
    """
    判断给定元素是否可用。

    参数:
        - 元素对象: 要进行可用性检查的元素对象。

    返回:
        - bool: 如果元素可用，则返回 True；否则返回 False。
    """
    try:
        return 元素对象.is_enabled()
    except Exception:
        return False


import sys


def 打印_打印进度条(当前值, 总值, 长度=40):
    """
    打印进度条。

    参数:
        - 当前值 (int): 当前进度值。
        - 总值 (int): 总进度值。
        - 长度 (int, optional): 进度条的长度。默认是 40。

    返回:
        - bool: 如果打印过程中没有出现异常，返回 True；否则返回 False。

    使用示例：
        总值 = 100
        for 当前值 in range(总值 + 1):
            打印_打印进度条(当前值, 总值)
            time.sleep(0.1)  # 模拟进度
    """
    try:
        百分比 = 当前值 / 总值
        完成长度 = int(长度 * 百分比)
        进度条 = "█" * 完成长度 + "-" * (长度 - 完成长度)
        sys.stdout.write(f"\r|{进度条}| {当前值}/{总值} ({百分比:.2%})")
        sys.stdout.flush()
        if 当前值 >= 总值:
            print()  # 完成后换行
        return True
    except Exception as e:
        print(f"打印时出现异常: {e}")
        return False



def 算数_求中位数(数值列表):
    """
    计算一组数的中位数。

    参数:
        - 数值列表 (list): 要计算中位数的数值列表。

    返回:
        - 中位数 (float): 数值列表的中位数。
    """
    try:
        数值列表.sort()
        n = len(数值列表)
        if n % 2 == 0:
            中位数 = (数值列表[n//2 - 1] + 数值列表[n//2]) / 2
        else:
            中位数 = 数值列表[n//2]
        return 中位数
    except Exception as e:
        print(f"计算中位数时发生错误: {e}")
        return None


def 编码_Rot13编码(字符串):
    """
    对字符串进行 Rot13 编码。

    参数:
        - 字符串 (str): 要进行编码的字符串。

    返回:
        - 编码后的字符串 (str)。
    """
    try:
        return 字符串.translate(str.maketrans(
            "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz",
            "NOPQRSTUVWXYZABCDEFGHIJKLMnopqrstuvwxyzabcdefghijklm"))
    except Exception:
        return None


def 文本文件_取指定行内容_取右边(文件路径, 行号, 分隔符):
    """
    获取指定文本文件中特定行的内容，并取分隔符右边的部分。

    注意：文本编码格式需要为UTF-8

    参数:
        - 文件路径 (str): 要读取的文本文件的路径。
        - 行号 (int): 要获取内容的行号（从1开始）。
        - 分隔符 (str): 用于分割行内容的分隔符。

    返回:
        - str: 分隔符右边的内容。失败返回 None。

    示例:
        行内容右边 = 文本文件_取指定行内容_取右边('示例.txt', 3, '----')
        if 行内容右边 is not None:
            print("行内容右边:", 行内容右边)
        else:
            print("获取行内容失败")
    """
    try:
        with open(文件路径, 'r', encoding='utf-8') as file:
            行内容 = file.readlines()

        if 行号 < 1 or 行号 > len(行内容):
            return None

        # 获取指定行并去除行尾的换行符
        指定行内容 = 行内容[行号 - 1].rstrip('\n')

        # 根据分隔符取分隔符右边的内容
        分隔符位置 = 指定行内容.find(分隔符)
        if 分隔符位置 == -1:
            return None

        右边内容 = 指定行内容[分隔符位置 + len(分隔符):]

        return 右边内容
    except Exception:
        return None


def 算数_求平均值(数值列表):
    """
    计算一组数的平均值。

    参数:
        - 数值列表 (list): 要计算平均值的数值列表。

    返回:
        - 平均值 (float): 数值列表的平均值。
    """
    try:
        平均值 = sum(数值列表) / len(数值列表)
        return 平均值
    except Exception as e:
        print(f"计算平均值时发生错误: {e}")
        return None


import math


def 算数_求标准差(数值列表):
    """
    计算一组数的标准差。

    参数:
        - 数值列表 (list): 要计算标准差的数值列表。

    返回:
        - 标准差 (float): 数值列表的标准差。如果输入无效或出现异常，返回 None 并打印错误信息。
    """
    try:
        # 计算平均值
        平均值 = sum(数值列表) / len(数值列表)

        # 计算方差
        方差 = sum((x - 平均值) ** 2 for x in 数值列表) / len(数值列表)

        # 计算标准差
        标准差 = math.sqrt(方差)

        return 标准差
    except Exception as e:
        print(f"计算标准差时发生错误: {e}")
        return None


import os


def 目录_取运行目录(加尾斜杠=False):
    """
    获取当前运行目录的函数。

    参数:
        - 加尾斜杠 (bool): 如果为 True，则在目录路径末尾添加一个反斜杠。默认为 False。

    返回:
        - str: 当前运行目录的路径，如果获取失败则返回 None。

    使用示例:
        运行目录 = 目录_取运行目录()
        print(运行目录)

        运行目录_加斜杠 = 目录_取运行目录(True)
        print(运行目录_加斜杠)
    """
    try:
        运行目录 = os.getcwd()  # 尝试获取当前运行目录
        return 运行目录 + '\\' if 加尾斜杠 else 运行目录
    except Exception:
        return None



def 列表_成员合并(列表, 连接符号):
    """
    将列表中的每个元素转换为字符串，并用指定的连接符号将它们连接起来。

    参数:
        - 列表 (list): 包含元素的列表
        - 连接符号 (str): 用于连接每个元素的字符串

    返回:
        - 合并后的字符串 (str): 所有元素连接而成的字符串。如果处理过程中发生异常，则返回错误信息。

    示例:
        结果 = 列表_成员合并([1, 2, 3], '-')
        print("合并后的字符串:", 结果)
    """
    try:
        # 使用连接符号连接列表中的每个元素，确保每个元素都转换为字符串
        合并后的字符串 = 连接符号.join(str(元素) for 元素 in 列表)
        return 合并后的字符串
    except Exception as e:
        # 捕获所有异常并返回错误信息
        return f"处理失败: {e}"


def 打印_分隔线(分隔符="=", 分割线长度=80):
    """
    打印一条分隔线。

    参数:
        - 分隔符 (str): 用于分隔线的字符。默认是 "="。
        - 分割线长度 (int): 分隔线的长度。默认是 80。
    """
    try:
        print(分隔符 * 分割线长度)
        return True
    except Exception as e:
        print(f"打印时出现异常: {e}")
        return False


import jsonpath
import json


def json_遍历指定层级指定键(数据对象, 层级路径, 键名):
    """
    使用说明：
    1、遍历指定层级指定键，按照指定的目录下开始进行寻找

    2、层级名称不固定可用*号代替。

    3、层级路径一定要写完整，最终一定要定位到自己想要的键去。类似于：数据[第一层][第二层][第三层]

    使用示例：zfx.json_遍历指定层级指定键(result, "data.wickedProductNoCache.*.edges", "*.shortId")


    参数:
    数据对象 (dict 或 str): Python 字典对象或 json 字符串。
    层级路径 (str): 路径部分。
    键名 (str): 要查找的键名。

    返回:
    list: 包含所有匹配键值的列表，如果没有匹配或异常则返回空列表。

    使用示例：
    结果 = json_遍历指定层级指定键(数据对象, 'data.第一层.第二层', '价格')
    """
    try:
        # 如果传入的是 json 字符串，则解析为 Python 字典
        if isinstance(数据对象, str):
            数据对象 = json.loads(数据对象)

        # 构建 JSONPath 表达式
        表达式 = f'$.{层级路径}[*].{键名}'

        # 执行 JSONPath 查询
        匹配的值 = jsonpath.jsonpath(数据对象, 表达式)

        return 匹配的值 if 匹配的值 else []
    except Exception:
        return []


import math


def 算数_求反正切(值):
    """
    计算给定值的反正切值。

    参数:
        - 值 (float): 要计算反正切值的数值。

    返回:
        - 角度 (float): 给定值的反正切值，返回的角度单位为度。如果输入无效或出现异常，返回 None 并打印错误信息。

    # 示例用法
    value = 1
    atan_value = 算数_求反正切(value)
    print(f"{value}的反正切值为: {atan_value} 度")
    """
    try:
        # 计算反正切值，结果为弧度
        弧度 = math.atan(值)

        # 将弧度转换为角度
        角度 = math.degrees(弧度)

        return 角度
    except Exception as e:
        print(f"计算反正切值时发生错误: {e}")
        return None



def 算数_取绝对值(数值):
    """
    取绝对值函数，返回给定数值的绝对值。

    参数:
        - 数值 (int or float): 需要取绝对值的数值。

    返回:
        - int or float: 给定数值的绝对值。如果输入无效或出现异常，返回 None 并打印错误信息。

    使用示例:
        结果 = 算数_取绝对值(5)  # 输出：5
        结果2 = 算数_取绝对值(-3)  # 输出：3
    """
    try:
        if 数值 >= 0:
            return 数值
        else:
            return -数值
    except Exception as e:
        print(f"取绝对值时发生错误: {e}")
        return None


def 谷歌填表_获取页面源代码(驱动器对象):
    """
    获取当前页面的源代码。

    参数:
        - 驱动器对象: WebDriver 对象。

    返回:
        - 当前页面的源代码，如果失败则返回 None。
    """
    try:
        return 驱动器对象.page_source
    except Exception:
        return None


def 邮件IMAP_检查连接状态(连接对象):
    """
    检查与IMAP服务器的连接状态。

    参数:
        连接对象 (IMAP4_SSL): IMAP4_SSL连接对象。

    返回:
        bool: 连接状态，True为连接正常，False为断开。

    示例:
        if 邮件IMAP_检查连接状态(邮箱):
            print("连接正常")
        else:
            print("连接已断开")
    """
    try:
        状态, _ = 连接对象.noop()
        return 状态 == 'OK'
    except Exception:
        return False



国家代码_中国 = "CN"  # 中国
国家代码_美国 = "US"  # 美国
国家代码_日本 = "JP"  # 日本
国家代码_德国 = "DE"  # 德国
国家代码_法国 = "FR"  # 法国
国家代码_英国 = "GB"  # 英国
国家代码_印度 = "IN"  # 印度
国家代码_意大利 = "IT"  # 意大利
国家代码_巴西 = "BR"  # 巴西
国家代码_俄罗斯 = "RU"  # 俄罗斯
国家代码_加拿大 = "CA"  # 加拿大
国家代码_澳大利亚 = "AU"  # 澳大利亚
国家代码_韩国 = "KR"  # 韩国
国家代码_墨西哥 = "MX"  # 墨西哥
国家代码_西班牙 = "ES"  # 西班牙
国家代码_南非 = "ZA"  # 南非
国家代码_新加坡 = "SG"  # 新加坡
国家代码_荷兰 = "NL"  # 荷兰
国家代码_瑞士 = "CH"  # 瑞士
国家代码_瑞典 = "SE"  # 瑞典
国家代码_阿根廷 = "AR"  # 阿根廷
国家代码_比利时 = "BE"  # 比利时
国家代码_智利 = "CL"  # 智利
国家代码_丹麦 = "DK"  # 丹麦
国家代码_芬兰 = "FI"  # 芬兰
国家代码_希腊 = "GR"  # 希腊
国家代码_匈牙利 = "HU"  # 匈牙利
国家代码_爱尔兰 = "IE"  # 爱尔兰
国家代码_以色列 = "IL"  # 以色列
国家代码_马来西亚 = "MY"  # 马来西亚
国家代码_挪威 = "NO"  # 挪威
国家代码_新西兰 = "NZ"  # 新西兰
国家代码_菲律宾 = "PH"  # 菲律宾
国家代码_波兰 = "PL"  # 波兰
国家代码_葡萄牙 = "PT"  # 葡萄牙
国家代码_罗马尼亚 = "RO"  # 罗马尼亚
国家代码_沙特阿拉伯 = "SA"  # 沙特阿拉伯
国家代码_泰国 = "TH"  # 泰国
国家代码_土耳其 = "TR"  # 土耳其
国家代码_阿联酋 = "AE"  # 阿联酋
国家代码_乌克兰 = "UA"  # 乌克兰
国家代码_越南 = "VN"  # 越南
国家代码_阿富汗 = "AF"  # 阿富汗
国家代码_阿尔巴尼亚 = "AL"  # 阿尔巴尼亚
国家代码_阿尔及利亚 = "DZ"  # 阿尔及利亚
国家代码_安道尔 = "AD"  # 安道尔
国家代码_安哥拉 = "AO"  # 安哥拉
国家代码_安提瓜和巴布达 = "AG"  # 安提瓜和巴布达
国家代码_亚美尼亚 = "AM"  # 亚美尼亚
国家代码_阿塞拜疆 = "AZ"  # 阿塞拜疆
国家代码_巴哈马 = "BS"  # 巴哈马
国家代码_巴林 = "BH"  # 巴林
国家代码_孟加拉国 = "BD"  # 孟加拉国
国家代码_巴巴多斯 = "BB"  # 巴巴多斯
国家代码_白俄罗斯 = "BY"  # 白俄罗斯
国家代码_伯利兹 = "BZ"  # 伯利兹
国家代码_贝宁 = "BJ"  # 贝宁
国家代码_不丹 = "BT"  # 不丹
国家代码_玻利维亚 = "BO"  # 玻利维亚
国家代码_波斯尼亚和黑塞哥维那 = "BA"  # 波斯尼亚和黑塞哥维那
国家代码_博茨瓦纳 = "BW"  # 博茨瓦纳
国家代码_文莱 = "BN"  # 文莱
国家代码_保加利亚 = "BG"  # 保加利亚
国家代码_布基纳法索 = "BF"  # 布基纳法索
国家代码_布隆迪 = "BI"  # 布隆迪



def 编码_字符串到十六进制文本_原文备注(text):
    """
    将字符串转换为16进制文本，并在每个字符后备注对应的原文字符。

    参数:
        text (str): 要转换的字符串。

    返回:
        str: 转换后的16进制文本，每个字符用两个字符表示，中间用空格分隔，并备注原文字符。如果输入无效或出现异常，返回 None。

    示例:
        text = "Hello, World!"
        hex_text = 编码_字符串到十六进制文本(text)
        print("字符串转换为16进制文本:", hex_text)  # 输出: 48(H) 65(e) 6c(l) 6c(l) 6f(o) 2c(,) 20( ) 57(W) 6f(o) 72(r) 6c(l) 64(d) 21(!)
    """
    try:
        byte_text = text.encode()
        hex_text_list = []
        i = 0
        while i < len(byte_text):
            # 检测当前字节是单字节字符还是多字节字符
            if byte_text[i] < 128:
                # 单字节字符
                char = chr(byte_text[i])
                hex_text_list.append(f"{byte_text[i]:02x}({char})")
                i += 1
            else:
                # 多字节字符，根据UTF-8编码进行处理
                if byte_text[i] & 0b11110000 == 0b11110000:
                    # 4字节字符
                    char_bytes = byte_text[i:i + 4]
                    char = char_bytes.decode()
                    hex_text_list.append(' '.join(f"{b:02x}" for b in char_bytes) + f"({char})")
                    i += 4
                elif byte_text[i] & 0b11100000 == 0b11100000:
                    # 3字节字符
                    char_bytes = byte_text[i:i + 3]
                    char = char_bytes.decode()
                    hex_text_list.append(' '.join(f"{b:02x}" for b in char_bytes) + f"({char})")
                    i += 3
                elif byte_text[i] & 0b11000000 == 0b11000000:
                    # 2字节字符
                    char_bytes = byte_text[i:i + 2]
                    char = char_bytes.decode()
                    hex_text_list.append(' '.join(f"{b:02x}" for b in char_bytes) + f"({char})")
                    i += 2

        hex_text = ' '.join(hex_text_list)
        return hex_text
    except Exception:
        return None


def 数据类型转换_到小数(输入值):
    """
    将输入值转换为小数（浮点数）。

    参数:
    输入值: 任何类型
        需要转换为小数的值。

    返回:
    float
        输入值转换后的小数。如果转换失败，则返回错误信息。
    """
    try:
        return float(输入值)
    except Exception as e:
        return e


def 数据类型转换_到None(输入值):
    """
    将任何输入值转换为 None。

    参数:
    输入值: 任何类型
        需要转换的值。

    返回:
    None
        返回 None。
    """
    try:
        # 这里直接返回 None，因为任何值都可以被“转换”成 None
        return None
    except Exception as e:
        return e


def 邮件POP3_获取邮件数量(服务器对象):
    """
    获取邮箱中的邮件数量。

    参数:
        - 邮件服务器 (poplib.POP3): 已经连接的POP3服务器对象。

    返回值:
        - int: 邮件数量。如果出现异常，返回假(False)。
    """
    try:
        # 获取邮箱中的邮件数量
        num_messages, _ = 服务器对象.stat()
        return num_messages
    except Exception:
        return False


import time
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By


def 谷歌填表_等待元素出现(驱动器对象, 定位方法, 定位值, 超时时间=30):
    """
    使用提供的WebDriver等待指定元素出现。

    参数:
        - 驱动器对象: WebDriver对象，用于控制浏览器的行为。
        - 定位方法: 可以是 "ID"、"XPATH"、"CLASS_NAME"、"NAME"、"TAG_NAME"、"LINK_TEXT"、"PARTIAL_LINK_TEXT"、"CSS_SELECTOR" 等。
        - 定位值: 元素的定位值。
        - 超时时间: 等待元素出现的最长时间，单位为秒，默认为 30 秒。

    返回:
        - WebElement: 如果元素出现，则返回该元素对象；如果超时未出现，则返回 None。
    """
    try:
        定位方法常量 = getattr(By, 定位方法.upper())
        元素对象 = WebDriverWait(驱动器对象, 超时时间).until(EC.presence_of_element_located((定位方法常量, 定位值)))
        time.sleep(1)
        return 元素对象
    except Exception:
        return None


def 文本文件_读入运行目录下文件内容(文件名称):
    """
    读取运行目录下的指定文本文件内容，文本文件需要和运行的Py文件在同一目录，只需要传递文本文件的名称就行了，不需要传递完整路径。

    参数:
        - 文件名称 (str): 要读取的文本文件名称。

    返回:
        - 文件内容 (str): 成功时返回文件内容，失败时返回 None。

    示例:
        文件内容 = 文本文件_读入运行目录下文件('example.txt')
        if 文件内容 is not None:
            print("文件内容读取成功:")
            print(文件内容)
        else:
            print("文件内容读取失败")
    """
    try:
        with open(文件名称, 'r', encoding='utf-8') as 文件:
            内容 = 文件.read()
        return 内容
    except Exception:
        return None


def mysql_查询_条件查找_排序(游标对象, 表名, 字段名, 排序方式="升序", 查询条件=""):
    """
    使用给定的游标对象执行查询，并返回符合条件的记录。

    参数：
        - 游标：数据库游标对象，用于执行查询。
        - 表名：要查询的表名。
        - 字段名：用于排序的字段名
        - 排序方式：可选，排序方式，默认为"升序"，可选值为"升序"或"降序"。
        - 查询条件：可选，查询条件，默认为空字符串。

    返回值：
        - 符合查询条件的记录，如果没有找到记录，则返回空列表。

    使用示例
    查询结果 = 数据库_mysql_查询记录(游标对象, "asida", "id", 排序方式="降序", 查询条件="id > 0")
    """
    try:
        # 构建查询语句
        查询语句 = f"SELECT * FROM {表名}"
        if 查询条件:
            查询语句 += f" WHERE {查询条件}"
        if 排序方式 == "升序":
            查询语句 += f" ORDER BY {字段名} ASC"
        elif 排序方式 == "降序":
            查询语句 += f" ORDER BY {字段名} DESC"

        # 执行查询
        游标对象.execute(查询语句)

        # 获取查询结果
        结果 = 游标对象.fetchall()

        return 结果
    except Exception:
        return False


def mysql_查询_第一条记录(连接对象, 表名):
    """
    查询指定表的第一条记录。

    参数：
        - 连接对象：与数据库的连接对象。
        - 表名：要查询第一条记录的表名。

    返回值：
        - 第一条记录：表中的第一条记录，如果查询失败则返回 None。
    """
    游标对象 = None
    try:
        游标对象 = 连接对象.cursor()
        游标对象.execute(f"SELECT * FROM {表名} LIMIT 1")
        第一条记录 = 游标对象.fetchone()
        return 第一条记录
    except Exception:
        return None
    finally:
        if 游标对象:
            游标对象.close()


def 打印_带标题打印列表(列表, 标题):
    """
    打印带标题的列表。

    参数:
        - 列表 (list): 要打印的列表。
        - 标题 (str): 列表的标题。

    返回:
        - bool: 如果打印过程中没有出现异常，返回 True；否则返回 False。
    """
    try:
        print(标题)
        print("=" * len(标题))
        for 元素 in 列表:
            print(元素)
        return True
    except Exception as e:
        print(f"打印时出现异常: {e}")
        return False


def mysql_增加_多条记录(连接对象, 表名, 数据列表):
    """
    添加多条记录到指定的表中。

    参数：
        - 连接对象: 数据库连接对象。
        - 表名: 需要添加记录的表的名称。
        - 数据列表: 包含多个数据字典的列表，每个数据字典包含字段和值，例如 [{"字段1": "值1", "字段2": "值2"}, ...]。

    返回值：
        - 添加成功返回 True，失败返回 False。
    """
    游标 = None
    try:
        # 构造字段列表
        字段列表 = ", ".join(数据列表[0].keys())
        # 构造值列表
        值列表 = ", ".join([f"({', '.join([f'\'{值}\'' for 值 in 数据.values()])})" for 数据 in 数据列表])

        # 构造完整的插入 SQL 语句
        sql = f"INSERT INTO {表名} ({字段列表}) VALUES {值列表};"

        # 获取游标对象，用于执行 SQL 语句
        游标 = 连接对象.cursor()
        # 执行 SQL 语句
        游标.execute(sql)
        # 提交事务，确保更改生效
        连接对象.commit()
        return True
    except Exception:
        return False
    finally:
        # 如果游标对象存在，关闭游标，释放资源
        if 游标:
            游标.close()


def 文本文件_追加内容(文件名, 内容):
    """
    向文本文件末尾追加内容。

    参数:
        - 文件名 (str): 要追加内容的文本文件名。
        - 内容 (str): 要追加的内容。

    返回:
        - 成功时返回 True，失败时返回 False。

    示例:
        追加成功 = 文本文件_追加内容('example.txt', '追加的内容\n')
        if 追加成功:
            print("内容追加成功")
        else:
            print("内容追加失败")
    """
    try:
        with open(文件名, 'a', encoding='utf-8') as 文件:
            文件.write(内容)
        return True
    except Exception:
        return False



def 文本文件_取行数(文件路径):
    """
    计算指定文本文件的行数。

    参数:
        文件路径 (str): 要计算行数的文本文件路径。

    返回:
        int: 文本文件内的行数。失败返回 -1。

    示例:
        行数 = 文本文件_取行数('示例.txt')
        if 行数 != -1:
            print(f"文件行数: {行数}")
        else:
            print("获取文件行数失败")
    """
    try:
        with open(文件路径, 'r', encoding='utf-8') as file:
            行数 = sum(1 for _ in file)
        return 行数
    except Exception:
        return -1


def 文本_取长度(源文本):
    """
    获取文本的长度。

    参数:
        源文本 (str): 要获取长度的文本。

    返回:
        int: 文本的长度，如果出现任何异常，则返回 -1。

    示例:
        文本 = "这是一段示例文本a"
        长度 = 文本_取长度(文本)
        print("文本的长度:", 长度)
    """
    try:
        return len(源文本)
    except Exception:  # 捕获所有异常
        return -1


def 文本文件_统计字符数(文件路径):
    """
    统计文件中的字符数。

    参数:
        - 文件路径 (str): 要统计的文件路径。

    返回:
        - 字符数 (int)。失败则返回 None
    """
    try:
        with open(文件路径, 'r', encoding='utf-8') as file:
            内容 = file.read()
            return len(内容)
    except Exception:
        return None


def 文本文件_取关键词出现次数(文件名, 关键词):
    """
    从指定的文本文件中获取特定关键词的出现次数。

    参数:
        文件名 (str): 要读取的文本文件的文件名。
        关键词 (str): 要查找并计数出现次数的关键词。

    返回值:
        int: 关键词在文件中出现的总次数。如果文件无法打开或处理出现错误，则返回 None。

    异常:
        如果文件无法打开或处理出现错误，将捕获异常并打印错误消息，然后返回 None。
    """
    try:
        出现次数 = 0
        with open(文件名, 'r', encoding='utf-8') as 文件:
            for 行 in 文件:
                出现次数 += 行.count(关键词)
        return 出现次数
    except Exception:
        return None


def excel_删除列(表格对象, 工作表名, 列号, 列数=1):
    """
    参数:
    - 表格对象: 要操作的表格对象
    - 工作表名: 要删除列的工作表名称
    - 列号: 要删除列的位置
    - 列数: 要删除的列数，默认为1

    返回值:
    - 如果成功删除列，则返回 True；如果删除失败，则返回 False
    """
    try:
        # 获取指定的工作表
        工作表 = 表格对象[工作表名]
        # 删除指定列数
        工作表.delete_cols(列号, 列数)
        return True
    except Exception:
        return False


import subprocess


def 模块_查看版本信息():
    """
    查看模块 zfx 的版本号。

    返回:
        str: 模块 zfx 的版本号，如果模块不存在或发生异常，则返回False。

    示例:
        version = a模块_查看版本信息()
        print("zfx的版本号:", version)
    """
    try:
        # 使用subprocess模块执行固定的命令
        subprocess.run(["pip", "show", "zfx"], check=True)
    except Exception:
        # 捕获其他所有异常并返回 False
        return False


def 列表_取元素出现次数(列表, 目标元素):
    """
    统计目标元素在列表中出现的次数。

    参数:
        - 列表 (list): 包含元素的列表
        - 目标元素: 要统计出现次数的目标元素

    返回:
        - 出现次数 (int): 目标元素在列表中出现的次数，如果出现异常则返回错误信息。
    """
    try:
        出现次数 = 0

        # 遍历列表中的每个元素
        for 元素 in 列表:
            # 如果元素等于目标元素，则计数加一
            if 元素 == 目标元素:
                出现次数 += 1

        return 出现次数
    except Exception as e:
        # 捕获所有异常并返回错误信息
        return f"处理失败: {e}"


def mysql_执行SQL语句(连接对象, SQL语句):
    """
    执行任意 SQL语句 并返回结果。

    参数：
        - 连接对象：与数据库的连接对象。
        - SQL语句：要执行的SQL查询字符串。

    返回值：
        - 结果：查询结果列表，如果查询失败则返回 None。
    """
    游标对象 = None
    try:
        # 创建游标对象，用于执行SQL查询
        游标对象 = 连接对象.cursor()
        # 执行SQL查询
        游标对象.execute(SQL语句)
        # 如果是查询语句，获取查询结果
        if SQL语句.strip().lower().startswith('select'):
            结果 = 游标对象.fetchall()
            return 结果
        else:
            # 对于插入、更新、删除操作，提交事务
            连接对象.commit()
            return 游标对象.rowcount  # 返回受影响的行数
    except Exception as e:
        print(f"执行SQL失败: {e}")
        return None
    finally:
        # 无论是否发生异常，关闭游标对象以释放资源
        if 游标对象:
            游标对象.close()


import threading


def 线程_取当前线程名称():
    """
    获取当前线程的名称

    返回值:
    - 当前线程的名称 (str)，如果获取失败则返回 False
    """
    try:
        当前线程 = threading.current_thread()
        return 当前线程.name
    except Exception:
        return False


import winreg


def 系统任务计划_遍历名称():
    """
    # 示例用法
    打印所有任务计划名称()
    """
    # 打开注册表键
    key_path = r"SOFTWARE\Microsoft\Windows NT\CurrentVersion\Schedule\TaskCache\Tree"
    try:
        with winreg.OpenKey(winreg.HKEY_LOCAL_MACHINE, key_path) as key:
            print("任务计划库中的任务名称：")
            # 枚举注册表子项
            for i in range(winreg.QueryInfoKey(key)[0]):
                task_name = winreg.EnumKey(key, i)
                print(task_name)
    except FileNotFoundError:
        print("找不到任务计划库的注册表项。")


def mysql_更新_SQL语句条件(连接对象, 表名, 值字典, 条件):
    """
    根据条件更新 MySQL 数据库表中的记录。

    参数：
        - 连接对象: 数据库连接对象。
        - 表名(字符串): 需要更新的表的名称。
        - 更新字段和值(字典): 要更新的字段及其对应的值，字典类型。例如 {"字段1": "值1", "字段2": "值2"}。
        - 条件(字符串): 用于筛选需要更新记录的条件，字符串类型。例如 "id = 1 AND name = 'Jack'"。

    返回值：
        - 更新成功返回 True，失败返回 False。
    """
    游标 = None
    try:
        # 生成更新字段和值的 SQL 片段
        # 使用列表解析将字典转换为 "字段 = '值'" 的形式，并用逗号分隔
        更新片段 = ", ".join([f"{字段} = '{值}'" for 字段, 值 in 值字典.items()])

        # 构造完整的更新 SQL 语句
        # 格式为：UPDATE 表名 SET 更新片段 WHERE 条件;
        sql = f"UPDATE {表名} SET {更新片段} WHERE {条件};"

        # 获取游标对象，用于执行 SQL 语句
        游标 = 连接对象.cursor()
        # 执行 SQL 语句
        游标.execute(sql)
        # 提交事务，确保更改生效
        连接对象.commit()
        return True
    except Exception:
        return False
    finally:
        # 如果游标对象存在，关闭游标，释放资源
        if 游标:
            游标.close()


import math


def 算数_求对数(值, 基数=math.e):
    """
    计算给定数值的对数。

    参数:
        - 值 (float): 要计算对数的数值。
        - 基数 (float): 对数的基数，默认是自然对数的基数e。

    返回:
        - 对数值 (float): 给定数值的对数。
    """
    try:
        对数值 = math.log(值, 基数)
        return 对数值
    except Exception as e:
        print(f"计算对数值时发生错误: {e}")
        return None


import json


def json_遍历_值_取键名(数据对象, 目标值):
    """
    查找 JSON 对象中指定值的所有键。

    参数:
        数据对象 (dict 或 str): Python 字典对象或 JSON 字符串。
        目标值 (any): 要查找的值。

    返回:
        list: 包含所有匹配值的键的列表，如果没有匹配或异常则返回空列表。

    使用示例：
        匹配结果 = zfx.json_值_取键名(JSON文本, 目标值)
        zfx.打印_依次打印列表(匹配结果)
    """
    try:
        if isinstance(数据对象, str):
            数据对象 = json.loads(数据对象)

        结果 = []

        def 遍历(obj):
            if isinstance(obj, dict):
                for k, v in obj.items():
                    if v == 目标值:
                        结果.append(k)  # 记录匹配值的键
                    if isinstance(v, (dict, list)):
                        遍历(v)
            elif isinstance(obj, list):
                for item in obj:  # 修正此处
                    if isinstance(item, (dict, list)):
                        遍历(item)

        遍历(数据对象)
        return 结果
    except Exception as e:
        return []


def 列表_插入元素(列表, 位置, 元素):
    """
    在指定位置插入一个元素到列表中。

    参数:
        - 列表 (list): 要插入元素的列表。
        - 位置 (int): 插入元素的位置索引。
        - 元素: 要插入的元素。

    返回:
        - list: 插入元素后的新列表。如果出现异常，返回原始列表。
    """
    try:
        # 使用列表的 insert 方法在指定位置插入元素
        列表.insert(位置, 元素)
        return 列表
    except Exception:
        return 列表


import math


def 算数_求余弧度(值):
    """
    计算反余弦、反正弦、反正切的值，返回结果为弧度。

    参数:
        - 值 (float): 要计算的数值。

    返回:
        - 余弧度值 (dict): 包含反余弦、反正弦、反正切的值。
    """
    try:
        反余弦 = math.acos(值)
        反正弦 = math.asin(值)
        反正切 = math.atan(值)
        return {
            "反余弧度": 反余弦,
            "反正弧度": 反正弦,
            "反正切弧度": 反正切
        }
    except Exception as e:
        print(f"计算余弧度值时发生错误: {e}")
        return None


import subprocess
import os
import tempfile


def 文件_封装Py代码成EXE_普通编译(脚本路径):
    """
    将指定的 Python 脚本打包成可执行的 EXE 文件，并将输出文件保存到桌面。

    参数:
    脚本路径 (str): 要打包的 Python 脚本的路径。该路径应指向一个有效的 Python 脚本文件。

    返回值:
    无

    异常:
    subprocess.CalledProcessError: 如果打包过程中出现错误，将会抛出 CalledProcessError 异常。

    注意:
    - 该函数将指定的 Python 脚本进行封装，将其转换为可执行的 EXE 文件。
    - 封装后的 EXE 文件将会保存在用户的桌面上。

    # 使用示例
    - 文件_封装Py代码成EXE_普通编译(r"F:\Python源码\aaaaa\CDK\abc.py")
    """
    try:
        # 获取系统桌面的路径
        桌面路径 = os.path.join(os.path.expanduser("~"), 'Desktop')

        # 创建一个临时目录
        with tempfile.TemporaryDirectory() as 临时目录:
            # 使用 subprocess.run 调用 pyinstaller 命令，将脚本打包成 EXE 文件，并指定输出目录为桌面
            subprocess.run(['pyinstaller', '--distpath', 桌面路径, '--workpath', 临时目录, '--specpath', 临时目录, 脚本路径], check=True)

        print(f"成功将 {脚本路径} 打包成 EXE 文件，并保存在桌面。")  # 打包成功后输出提示信息
    except subprocess.CalledProcessError as e:
        # 如果打包过程中发生错误，捕获异常并输出错误信息
        print(f"打包过程中出现错误: {e}")


def 算数_求余数(被除数, 除数):
    """
    计算被除数除以除数的余数。

    参数:
        被除数 (int): 被除数。
        除数 (int): 除数。

    返回:
        int: 余数。如果输入无效或出现异常，返回 None 并打印错误信息。

    使用示例:
        print(算数_求余数(10, 3))  # 输出：1
        print(算数_求余数(7, 4))   # 输出：3
    """
    try:
        return 被除数 % 除数
    except Exception as e:
        print(f"计算余数时发生错误: {e}")
        return None


import time
from selenium.webdriver.common.by import By


def 谷歌填表_查找元素(驱动器对象, 定位方法, 定位值):
    """
    在网页上查找元素。

    参数:
        - 驱动器对象: WebDriver 对象，即浏览器驱动器，用于在网页上执行操作。
        - 定位方法: 可以是 "ID"、"XPATH"、"CLASS_NAME"、"NAME"、"TAG_NAME"、"LINK_TEXT"、"PARTIAL_LINK_TEXT"、"CSS_SELECTOR" 等。
        - 定位值: 定位值，根据定位方法指定的方式，传入相应的定位值。

    返回:
        - 如果找到元素，则返回查找到的第一个元素对象，否则返回 None。
    """
    try:
        定位方法常量 = getattr(By, 定位方法.upper())
        元素对象 = 驱动器对象.find_element(定位方法常量, value=定位值)
        time.sleep(1)
        return 元素对象
    except Exception:
        return None


def 谷歌填表_置浏览器大小和位置(驱动器对象, 宽度, 高度, x_位置, y_位置):
    """
    设置浏览器窗口的大小和位置。

    参数:
        - 驱动器对象: WebDriver 对象，表示浏览器驱动器。
        - 宽度: int，表示窗口宽度（像素）。
        - 高度: int，表示窗口高度（像素）。
        - x_位置: int，表示窗口左上角的 x 坐标位置。
        - y_位置: int，表示窗口左上角的 y 坐标位置。

    返回:
        - bool: 设置成功返回 True，失败返回 False。
    """
    try:
        驱动器对象.set_window_size(宽度, 高度)
        驱动器对象.set_window_position(x_位置, y_位置)
        return True
    except Exception:
        return False


def 文本_去除指定字符串_自定义位置(原始文本: str, 目标文本: str, 出现次数: int):
    """
    去除原始文本中目标文本的第几次出现。

    参数:
        - 原始文本(str): 原始的文本内容。
        - 目标文本(str): 需要去除的字符或字符串。
        - 出现次数(int): 需要去除的是第几次出现。

    返回:
        处理后的文本。如果次数不足或出现异常则返回原始文本
    """
    try:
        位置 = -1

        # 找到目标文本的第几次出现位置
        for i in range(出现次数):
            位置 = 原始文本.find(目标文本, 位置 + 1)
            if 位置 == -1:
                # 如果目标文本出现的次数不足，返回原始文本
                return 原始文本

        # 去除目标文本的第几次出现
        if 位置 != -1:
            return 原始文本[:位置] + 原始文本[位置 + len(目标文本):]
        return 原始文本
    except Exception:
        return 原始文本


def 谷歌填表_清除文本(元素对象):
    """
    清除元素对象中的文本。

    参数:
        - 元素对象: 要清除文本的 WebElement 对象。

    返回:
        - bool: 清除文本成功返回 True，失败返回 False。
    """
    try:
        元素对象.clear()
        return True
    except Exception:
        return False


def 文本_取左边(被查找的文本, 欲寻找的文本, 起始寻找位置=1, 是否不区分大小写=False):
    """
    从被查找的文本中寻找指定的文本，并返回该文本左边的所有字符。如果找不到，返回空字符串。

    参数:
        - 被查找的文本 (str): 要在其中进行查找的文本。
        - 欲寻找的文本 (str): 需要寻找的目标文本。
        - 起始寻找位置 (int, optional): 开始查找的位置索引，默认为 1。正数表示从左往右查找，负数表示从右往左查找。
        - 是否不区分大小写 (bool, optional): 如果为 True，查找时不区分大小写；如果为 False，区分大小写。默认值为 False。

    返回:
        - str: 欲寻找的文本左边的文本内容。如果找不到或出现异常，返回空字符串。

    示例用法:
        文本 = "这是一个示例文本，包含要寻找的文本"
        欲寻找的文本 = "示例"
        左边文本 = 文本_取左边(文本, 欲寻找的文本)
        print("欲寻找文本左边的内容:", 左边文本)
    """
    try:
        if 是否不区分大小写:
            被查找的文本 = 被查找的文本.lower()
            欲寻找的文本 = 欲寻找的文本.lower()

        if 起始寻找位置 < 0:
            被查找的文本 = 被查找的文本[::-1]
            欲寻找的文本 = 欲寻找的文本[::-1]

        index = 被查找的文本.find(欲寻找的文本, abs(起始寻找位置) - 1)
        if index != -1:
            if 起始寻找位置 < 0:
                return 被查找的文本[:index][::-1]
            else:
                return 被查找的文本[:index]
        else:
            return ""
    except Exception:
        return ""


def mysql_关闭连接(连接对象, 游标对象=None):
    """
    关闭数据库连接和游标对象。

    参数：
        - 连接: 数据库连接对象。
        - 游标对象：可选，数据库游标对象。如果提供了游标对象，则同时关闭游标对象；如果未提供，则仅关闭连接对象。

    返回值：
        - 成功返回 True，失败返回 False
    """
    try:
        # 关闭游标对象（如果提供）
        if 游标对象:
            游标对象.close()

        # 关闭连接对象
        连接对象.close()
        return True
    except Exception:
        return False


def 文本文件_删除指定行(文件路径, 行号):
    """
    删除指定行的内容。

    参数:
        - 文件路径 (str): 要操作的文件路径。
        - 行号 (int): 要删除的行号。

    返回:
        - bool: 如果删除成功，返回 True；否则返回 False。
    """
    try:
        with open(文件路径, 'r+', encoding='utf-8') as file:
            行列表 = file.readlines()
            if 0 < 行号 <= len(行列表):
                行列表.pop(行号 - 1)
                file.seek(0)
                file.writelines(行列表)
                file.truncate()
                return True
            return False
    except Exception:
        return False


import math


def 算数_向上取整(数字):
    """
    向上取整数，返回不小于给定数字的最小整数。

    参数:
        数字 (float): 需要向上取整的数字。

    返回:
        int: 不小于给定数字的最小整数。如果失败则返回 False。

    使用示例:
        结果 = 算数_向上取整(3.4)
        print("结果:", 结果)  # 输出结果为 4
    """
    try:
        return math.ceil(数字)
    except Exception:
        return False


def 谷歌填表_访问网页(驱动器对象, 网址):
    """
    使用提供的驱动程序访问指定的网址。

    参数:
        - 驱动器对象: WebDriver 对象，用于控制浏览器的行为。
        - 网址: 要访问的网址。

    返回:
        - bool: 访问成功返回 True，访问失败返回 False。
    """
    try:
        驱动器对象.get(网址)
        return True
    except Exception:
        return False


import os


def 目录_取桌面目录(加尾斜杠=False):
    """
    获取当前用户的桌面目录路径。

    参数:
        - 加尾斜杠 (bool): 如果为 True，则在目录路径末尾添加一个反斜杠。默认为 False。

    返回:
        - str: 当前用户的桌面目录路径。如果获取失败或异常则返回 None。

    示例:
        桌面目录 = 目录_取桌面目录()
        if 桌面目录 is not None:
            print("桌面目录路径:", 桌面目录)
        else:
            print("获取桌面目录失败")

        桌面目录_加斜杠 = 目录_取桌面目录(True)
        if 桌面目录_加斜杠 is not None:
            print("桌面目录路径(加斜杠):", 桌面目录_加斜杠)
        else:
            print("获取桌面目录失败")
    """
    try:
        if os.name == 'nt':  # Windows
            桌面目录 = os.path.join(os.environ['USERPROFILE'], 'Desktop')
        else:  # macOS 和 Linux
            桌面目录 = os.path.join(os.path.expanduser('~'), 'Desktop')

        if os.path.exists(桌面目录):
            return 桌面目录 + '\\' if 加尾斜杠 else 桌面目录
        else:
            return None
    except Exception:
        return None


import urllib.parse


def 编码_URL编码(字符串):
    """
    对字符串进行 URL 编码。

    参数:
        - 字符串 (str): 要进行编码的字符串。

    返回:
        - 编码后的字符串 (str)。发生异常则返回None
    """
    try:
        编码后的字符串 = urllib.parse.quote(字符串)
        return 编码后的字符串
    except Exception:
        return None


def 列表_反转(列表):
    """
    反转列表中的元素顺序。

    参数:
        - 列表 (list): 要反转的列表。

    返回:
        - list: 反转后的列表。如果出现异常，返回原始列表。
    """
    try:
        return 列表[::-1]
    except Exception:
        return 列表


def excel_插入行(表格对象, 工作表名, 行号):
    """
    参数:
    - 表格对象: 要操作的表格对象
    - 工作表名: 要插入行的工作表名称
    - 行号: 要插入行的行号

    返回值:
    - 如果成功插入行，则返回 True；如果插入失败，则返回 False
    """
    try:
        工作表 = 表格对象[工作表名]
        工作表.insert_rows(行号)
        return True
    except Exception:
        return False


def 列表_按元素数量分组(列表, 组大小):
    """
    将列表按指定大小分组。

    参数:
        - 列表 (list): 要分组的列表。
        - 组大小 (int): 每个组的大小。

    返回:
        - list: 包含分组后的子列表的列表。如果出现异常，返回空列表。
    """
    try:
        # 使用列表推导式和切片进行分组
        分组结果 = [列表[i:i + 组大小] for i in range(0, len(列表), 组大小)]
        return 分组结果
    except Exception:
        return []


import sys
import traceback


class SimpleDebugger:
    def __init__(self):
        self.breakpoints = []

    def set_breakpoint(self, line, message=""):
        self.breakpoints.append((line, message))

    def start(self):
        frame = sys._getframe(1)  # 获取调用者的frame
        while frame:
            lineno = frame.f_lineno
            for bp in self.breakpoints:
                if bp[0] == lineno:
                    print(f"调试信息: {bp[1]} (在第 {lineno} 行)，指令提示：输入c将继续执行，输入q将退出代码运行")
                    self.interactive_console(frame)
            frame = frame.f_back

    def interactive_console(self, frame):
        local_vars = frame.f_locals
        global_vars = frame.f_globals
        while True:
            try:
                command = input("指令：")
                if command.strip() in ["c", "continue"]:
                    return
                elif command.strip() in ["q", "quit"]:
                    sys.exit(0)
                else:
                    exec(command, global_vars, local_vars)
            except Exception as e:
                print(f"Error: {e}")
                traceback.print_exc()


# 创建一个全局的调试器实例
global_debugger = SimpleDebugger()


def 调试_断点(信息="自定义断点信息"):
    """
    设置一个断点并启动调试器。

    参数:
        - 信息 (str): 自定义的调试信息，默认是"自定义断点信息"。
    """
    frame = sys._getframe(1)
    global_debugger.set_breakpoint(frame.f_lineno, 信息)
    global_debugger.start()


import csv


def mysql_查询_导出数据表(连接对象, 表名, 文件路径):
    """
    导出整个表的数据到本地文件（例如 CSV 文件）不建议大表使用，会等待很久。

    参数：
        - 连接对象: 数据库连接对象。
        - 表名: 需要导出的表的名称。
        - 文件路径: 导出文件的本地路径（例如 '数据.csv'）。

    返回值：
        - 导出成功返回 True，失败返回 False。
    """
    游标 = None
    try:
        # 获取游标对象，用于执行 SQL 查询
        游标 = 连接对象.cursor()
        # 执行查询语句，获取所有数据
        游标.execute(f"SELECT * FROM {表名};")
        结果 = 游标.fetchall()

        # 获取表的列名
        列名 = [i[0] for i in 游标.description]

        # 将数据写入 CSV 文件
        with open(文件路径, 'w', newline='', encoding='utf-8') as 文件:
            写入器 = csv.writer(文件)
            写入器.writerow(列名)
            写入器.writerows(结果)

        return True
    except Exception:
        return False
    finally:
        # 如果游标对象存在，关闭游标，释放资源
        if 游标:
            游标.close()